<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use React\Promise;
use Bunny\Channel;

use InSegment\WorkersFramework\Drivers\DeclareBuilderBase;

class DeclareBuilder extends DeclareBuilderBase
{
    
    /**
     * Bunny channel
     *
     * @var \Bunny\Channel
     */
    protected $channel;
    
    /**
     * Booleans. Arguments to the queueDeclare
     */
    protected $passive = false;
    protected $durable = true;
    protected $exclusive = false;
    protected $autoDelete = false;
    protected $nowait = false;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Drivers\RabbitMq\Channel $channel
     */
    public function __construct(Channel $channel)
    {
        $this->channel = $channel;
    }

    /**
     * {@inheritDoc}
     */
    public function doDeclare(bool $delete = false): Promise\PromiseInterface
    {
        if (!$this->queueName) {
            return Promise\reject(new \Exception('An attempt to declare an anonymous queue'));
        }
            
        if ($delete) {
            return $this->channel->queueDelete($this->queueName)->then(function() {
                return $this->channel->queueDeclare(
                    $this->queueName,
                    $this->passive,
                    $this->durable,
                    $this->exclusive,
                    $this->autoDelete,
                    $this->nowait,
                    $this->buildArguments()
                );
            });
        }
        
        return $this->channel->queueDeclare(
            $this->queueName,
            $this->passive,
            $this->durable,
            $this->exclusive,
            $this->autoDelete,
            $this->nowait,
            $this->buildArguments()
        );
    }
    
    /**
     * Get x-arguments (last argument to queueDeclare)
     * 
     * @return array
     */
    protected function buildArguments(): array
    {
        $arguments = [];
        
        if ($this->dlxQueueName !== null) {
            $arguments['x-dead-letter-exchange'] = '';
            $arguments['x-dead-letter-routing-key'] = $this->dlxQueueName;
        }
        
        if ($this->messageTtl !== null) {
            $arguments['x-message-ttl'] = $this->messageTtl;
        }
        
        return $arguments;
    }
}
