<?php

namespace InSegment\WorkersFramework\Http\Controllers;

use Shavshukov\EventAppBase\Http\Controller;
use Psr\Http\Message\ServerRequestInterface;
use React\Http\Response;
use React\Promise;

use InSegment\WorkersFramework\Services\Manager;

class AnnounceController extends Controller
{
    /**
     * Modification of state and on/off workers
     * 
     * @param \Psr\Http\Message\ServerRequestInterface $request
     * @return \React\Promise\PromiseInterface
     */
    public function postAnnounce(ServerRequestInterface $request): Promise\PromiseInterface
    {
        return $this->app->service('manager')->then(function (Manager $manager) use ($request) {
            $body = $request->getParsedBody();

            if (!isset($body['queue'])) {  //  lock/unlock all
                if ($body['queue-lock'] === 'true') {
                    return $manager->deactivateConsuming();
                } else if ($body['queue-lock'] === 'false') {
                    return $manager->activateConsuming();
                }
            } else {  //  lock/unlock specific worker
                if ($body['queue-lock'] === 'true') {
                    return $manager->deactivateWorker($body['queue']);
                } else if ($body['queue-lock'] === 'false') {
                    return $manager->activateWorker($body['queue']);
                }
            }
        })->then(function () {
            return new Response(200, [], 'ok');
        });
    }
}
