<?php

namespace InSegment\WorkersFramework\Queues;

use React\Promise\PromiseInterface;

use InSegment\WorkersFramework\Interfaces\QueueInterface;
use InSegment\WorkersFramework\Interfaces\ConsumerInterface;
use InSegment\WorkersFramework\Interfaces\QueueDriverInterface;

class Queue implements QueueInterface
{
    
    /**
     * Connection instance
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueDriverInterface
     */
    protected $connection;
    
    /**
     * Queue name
     *
     * @var string
     */
    protected $name;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueDriverInterface $connection
     * @param string $queueName
     */
    public function __construct(QueueDriverInterface $connection, string $queueName)
    {
        $this->connection = $connection;
        $this->name = $queueName;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getConnection(): QueueDriverInterface
    {
        return $this->connection;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getQueueName(): string
    {
        return $this->name;
    }
    
    /**
     * {@inheritDoc}
     */
    public function subscribe(ConsumerInterface $reciever): PromiseInterface
    {
        return $this->connection->subscribe($this->name, $reciever);
    }
    
    /**
     * {@inheritDoc}
     */
    public function unsubscribe(string $consumerTag): PromiseInterface
    {
        return $this->connection->unsubscribe($consumerTag);
    }
    
}
